## OpenXPKI::Server::API.pm 
##
## Written 2005, 2006 by Michael Bell, Martin Bartosch and
## Alexander Klink for the OpenXPKI project
## restructured 2006 by Alexander Klink for the OpenXPKI project
## Copyright (C) 2005-2006 by The OpenXPKI Project
## $Revision: 543 $

package OpenXPKI::Server::API;

use strict;
use warnings;
use utf8;
use English;

use Class::Std;

use Data::Dumper;

use Regexp::Common;
use Params::Validate qw( validate :types );

use OpenXPKI::Debug 'OpenXPKI::Server::API';
use OpenXPKI::Exception;
use OpenXPKI::Server::Context qw( CTX );
use OpenXPKI::DN;
use OpenXPKI::Server::API::Default;
use OpenXPKI::Server::API::Object;
use OpenXPKI::Server::API::Visualization;
use OpenXPKI::Server::API::Workflow;

my %external_of    :ATTR;
my %method_info_of :ATTR;

sub BUILD {
    my ($self, $ident, $arg_ref) = @_;
    
    Params::Validate::validation_options(
	# let parameter validation errors throw a proper exception
	on_fail => sub {
	    my $error = shift;
	    
	    OpenXPKI::Exception->throw (
		message => "I18N_OPENXPKI_SERVER_API_INVALID_PARAMETER",
		params => {
		    ERROR => $error,
		});
	},
    );
    if ($arg_ref->{EXTERNAL}) { # we are called externally, do ACL checks
        $external_of{$ident} = 1;
    }

    my $re_alpha_string      = qr{ \A [ \w \- \. : \s ]* \z }xms;
    my $re_integer_string    = qr{ \A $RE{num}{int} \z }xms;
    my $re_base64_string     = qr{ \A [A-Za-z0-9\+/=]* \z }xms;
    my $re_image_format      = qr{ \A (ps|png|jpg|gif|cmapx|imap|svg|svgz|mif|fig|hpgl|pcl|NULL) \z }xms;
    $method_info_of{$ident} = {
        ### Default API
        'get_pki_realm' => {
            class  => 'Default',
            params => { },
        },
        'get_user' => {
            class  => 'Default',
            params => { },
        },
        'get_role' => {
            class  => 'Default',
            params => { },
        },
        'get_chain' => {
            class  => 'Default',
            params => {
                START_IDENTIFIER => {
                    type     => SCALAR,
                    regex    => $re_base64_string,
                }, 
            },
        },
        'get_ca_certificate' => {
            class  => 'Default',
            params => { },
        },
        'list_ca_ids' => {
            class  => 'Default',
            params => { },
        },
        'get_pki_realm_index' => { # TODO: find out if this is actually used
                                   # externally or if it is only an internal
                                   # helper function
            class  => 'Default',
            params => { },
        },
        'get_roles' => {
            class  => 'Default',
            params => { },
        },
        'get_cert_profiles' => {
            class  => 'Default',
            params => { },
        },
        'get_cert_subject_profiles' => {
            class  => 'Default',
            params => {
                PROFILE => {
                    type  => SCALAR,
                    regex => $re_alpha_string,
                },
            },
        },

        ### Object API
        'get_csr_info_hash_from_data' => {
            class  => 'Object',
            params => {
                DATA => {
                    type     => SCALAR, # TODO: regexp?
                },
            },
        },

        ### Visualization API
        'get_workflow_instance_info' => {
            class  => 'Visualization',
            params => {
                ID => {
                    type  => SCALAR,
                    regex => $re_integer_string,
                },
                FORMAT => {
                    type  => SCALAR,
                    regex => $re_image_format,
                },
                LANGUAGE => {
                    type => SCALAR, # TODO: regexp?
                },
            },
        },

        ### Workflow API
        'list_workflow_instances' => {
            class  => 'Workflow',
            params => { },
        },
        'list_workflow_titles' => {
            class  => 'Workflow',
            params => { },
        },
        'get_workflow_info' => {
            class  => 'Workflow',
            params => {
                WORKFLOW => {
                    type  => SCALAR,
                    regex => $re_alpha_string,
                },
	        ID => {
		    type  => SCALAR,
		    regex => $re_integer_string,
                },
	    },
        },
        'execute_workflow_activity' => {
            class  => 'Workflow',
            params => {
                WORKFLOW => {
                    type  => SCALAR,
                    regex => $re_alpha_string,
                },
	        ID => {
		    type  => SCALAR,
		    regex => $re_integer_string,
                },
                ACTIVITY => {
                    type  => SCALAR,
                    regex => $re_alpha_string,
                },
                PARAMS   => {
                    type     => HASHREF,
                    optional => 1,
                },
            },
        },
        'create_workflow_instance' => {
            class  => 'Workflow',
            params => {
                WORKFLOW => {
                    type  => SCALAR,
                    regex => $re_alpha_string,
                },
                FILTER_PARAMS => {
                    type    => SCALAR,
                    regex   => $re_alpha_string,
                    default => 0,
                },
                PARAMS => {
                    type     => HASHREF,
                    optional => 1,
                },
            },
        },
        'get_workflow_activities' => {
            class  => 'Workflow',
            params => {
                WORKFLOW => {
                    type  => SCALAR,
                    regex => $re_alpha_string,
                },
	        ID => {
		    type  => SCALAR,
		    regex => $re_integer_string,
                },
	    },
        },
        'search_workflow_instances' => {
            class  => 'Workflow',
            params => {
                CONTEXT => {
                    type => HASHREF,
                },
                TYPE => {
                    type     => SCALAR,
                    regex    => $re_alpha_string,
                    optional => 1,
                },
            },
        },
    };
}

sub AUTOMETHOD {
    my ($self, $ident, @args) = @_;

    my $method_name = $_;

    ##! 16: 'method name: ' . $method_name
    return sub {
        if (!exists $method_info_of{$ident}->{$method_name}) {
            ##! 16: 'exception'
            OpenXPKI::Exception->throw(
                message => 'I18N_OPENXPKI_SERVER_API_INVALID_METHOD_CALLED',
                params  => {
                    'METHOD_NAME' => $method_name,
                },
            );
        }
        my $class         = $method_info_of{$ident}->{$method_name}->{class};
        my $valid_params  = $method_info_of{$ident}->{$method_name}->{params};
    
        ##! 16: 'args: ' . Dumper(\@args)
        # do parameter checking
        if (scalar @args > 1 || defined $args[0]) {
            validate(
    	        @args,
                $valid_params,
            );
        }
        
        # ACL checking
        if ($external_of{$ident}) { # do ACL checking
            my $affected_role
                = $method_info_of{$ident}->{$method_name}->{affected_role};
            my $acl_hashref = {
                ACTIVITY      => "API::" . $class . "::" . $method_name,
            };
            if (defined $affected_role) {
                # FIXME: optional for now, as we don't have a good way to
                # figure out which role is affected by a certain API call
                $acl_hashref->{AFFECTED_ROLE} = $affected_role;
            }
            eval {
                CTX('acl')->authorize($acl_hashref);
            };
        
            if (my $exc = OpenXPKI::Exception->caught()) {
                OpenXPKI::Exception->throw(
                    message => 'I18N_OPENXPKI_SERVER_API_ACL_CHECK_FAILED',
                    params  => {
                        'EXCEPTION' => $exc->message(),
                        'PARAMS'    => $exc->params(),
                    },
                );
            }
            elsif ($EVAL_ERROR) {
                OpenXPKI::Exception->throw(
                    message => 'I18N_OPENXPKI_SERVER_API_ACL_CHECK_FAILED',
                    params  => {
                        'EVAL_ERROR' => $EVAL_ERROR,
                    },
                );
            }
        }
    
        # call corresponding method
        my $openxpki_class = "OpenXPKI::Server::API::" . $class;
        return $openxpki_class->$method_name(@args);
    };
}
    
###########################################################################

1;
__END__

=head1 Name

OpenXPKI::Server::API

=head1 Description

This is the interface which should be used by all user interfaces of OpenXPKI.
A user interface MUST NOT access the server directly. The only allowed
access is via this API. Any function which is not available in this API is
not for public use.
The API gets access to the server via the 'server' context object. This
object must be set before instantiating the API.

=head1 Functions

=head2 new

Default constructor created by Class::Std. The named parameter
EXTERNAL should be set to 1 if the API is used from an external
source (e.g. within a service). If EXTERNAL is set, in addition to
the parameter checks, ACL checks are enabled.

=head2 AUTOMETHOD

This method is used by Class::Std when a method is called that is undefined
in the current class. In our case, this method does the parameter
checking for the requested method. If the class has been instantiated
with the EXTERNAL parameter, ACL checks are done in addition. Then the
class name is constructed from the $method_info hash reference and
the corresponding method is called with the given parameters.
